@@ -9,14 +9,13 @@ import (
 
 	"golang.org/x/tools/go/analysis/analysistest"
 	"golang.org/x/tools/go/analysis/passes/printf"
-	"golang.org/x/tools/internal/testenv"
 )
 
 func Test(t *testing.T) {
-	testenv.NeedsGo1Point(t, 19) // tests use fmt.Appendf
-
 	testdata := analysistest.TestData()
 	printf.Analyzer.Flags.Set("funcs", "Warn,Warnf")
 
-	analysistest.Run(t, testdata, printf.Analyzer, "a", "b", "nofmt", "typeparams")
+	analysistest.Run(t, testdata, printf.Analyzer,
+		"a", "b", "nofmt", "typeparams", "issue68744")
+	analysistest.RunWithSuggestedFixes(t, testdata, printf.Analyzer, "fix")
 }
