@@ -28,16 +28,16 @@ import (
 	"golang.org/x/tools/gopls/internal/cache/typerefs"
 	"golang.org/x/tools/gopls/internal/file"
 	"golang.org/x/tools/gopls/internal/filecache"
+	"golang.org/x/tools/gopls/internal/label"
 	"golang.org/x/tools/gopls/internal/protocol"
+	"golang.org/x/tools/gopls/internal/tokeninternal"
 	"golang.org/x/tools/gopls/internal/util/bug"
 	"golang.org/x/tools/gopls/internal/util/safetoken"
 	"golang.org/x/tools/gopls/internal/util/slices"
 	"golang.org/x/tools/internal/analysisinternal"
 	"golang.org/x/tools/internal/event"
-	"golang.org/x/tools/internal/event/tag"
 	"golang.org/x/tools/internal/gcimporter"
 	"golang.org/x/tools/internal/packagesinternal"
-	"golang.org/x/tools/internal/tokeninternal"
 	"golang.org/x/tools/internal/typesinternal"
 	"golang.org/x/tools/internal/versions"
 )
@@ -55,14 +55,10 @@ type unit = struct{}
 // It shares state such as parsed files and imports, to optimize type-checking
 // for packages with overlapping dependency graphs.
 type typeCheckBatch struct {
-	activePackageCache interface {
-		getActivePackage(id PackageID) *Package
-		setActivePackage(id PackageID, pkg *Package)
-	}
 	syntaxIndex map[PackageID]int // requested ID -> index in ids
 	pre         preTypeCheck
 	post        postTypeCheck
-	handles     map[PackageID]*packageHandle
+	handles     map[PackageID]*packageHandle // (immutable)
 	parseCache  *parseCache
 	fset        *token.FileSet // describes all parsed or imported files
 	cpulimit    chan unit      // concurrency limiter for CPU-bound operations
@@ -169,7 +165,10 @@ func (s *Snapshot) getImportGraph(ctx context.Context) *importGraph {
 			defer release()
 			importGraph, err := s.resolveImportGraph() // may be nil
 			if err != nil {
-				if ctx.Err() == nil {
+				// resolveImportGraph operates on the background context, because it is
+				// a shared resource across the snapshot. If the snapshot is cancelled,
+				// don't log an error.
+				if s.backgroundCtx.Err() == nil {
 					event.Error(ctx, "computing the shared import graph", err)
 				}
 				importGraph = nil
@@ -344,20 +343,55 @@ type (
 //
 // Both pre and post may be called concurrently.
 func (s *Snapshot) forEachPackage(ctx context.Context, ids []PackageID, pre preTypeCheck, post postTypeCheck) error {
-	ctx, done := event.Start(ctx, "cache.forEachPackage", tag.PackageCount.Of(len(ids)))
+	ctx, done := event.Start(ctx, "cache.forEachPackage", label.PackageCount.Of(len(ids)))
 	defer done()
 
-	if len(ids) == 0 {
+	var (
+		needIDs []PackageID // ids to type-check
+		indexes []int       // original index of requested ids
+	)
+
+	// Check for existing active packages.
+	//
+	// Since gopls can't depend on package identity, any instance of the
+	// requested package must be ok to return.
+	//
+	// This is an optimization to avoid redundant type-checking: following
+	// changes to an open package many LSP clients send several successive
+	// requests for package information for the modified package (semantic
+	// tokens, code lens, inlay hints, etc.)
+	for i, id := range ids {
+		if pkg := s.getActivePackage(id); pkg != nil {
+			post(i, pkg)
+		} else {
+			needIDs = append(needIDs, id)
+			indexes = append(indexes, i)
+		}
+	}
+
+	if len(needIDs) == 0 {
 		return nil // short cut: many call sites do not handle empty ids
 	}
 
-	handles, err := s.getPackageHandles(ctx, ids)
+	// Wrap the pre- and post- funcs to translate indices.
+	var pre2 preTypeCheck
+	if pre != nil {
+		pre2 = func(i int, ph *packageHandle) bool {
+			return pre(indexes[i], ph)
+		}
+	}
+	post2 := func(i int, pkg *Package) {
+		s.setActivePackage(pkg.metadata.ID, pkg)
+		post(indexes[i], pkg)
+	}
+
+	handles, err := s.getPackageHandles(ctx, needIDs)
 	if err != nil {
 		return err
 	}
 
 	impGraph := s.getImportGraph(ctx)
-	_, err = s.forEachPackageInternal(ctx, impGraph, nil, ids, pre, post, handles)
+	_, err = s.forEachPackageInternal(ctx, impGraph, nil, needIDs, pre2, post2, handles)
 	return err
 }
 
@@ -367,16 +401,15 @@ func (s *Snapshot) forEachPackage(ctx context.Context, ids []PackageID, pre preT
 // If a non-nil importGraph is provided, imports in this graph will be reused.
 func (s *Snapshot) forEachPackageInternal(ctx context.Context, importGraph *importGraph, importIDs, syntaxIDs []PackageID, pre preTypeCheck, post postTypeCheck, handles map[PackageID]*packageHandle) (*typeCheckBatch, error) {
 	b := &typeCheckBatch{
-		activePackageCache: s,
-		pre:                pre,
-		post:               post,
-		handles:            handles,
-		parseCache:         s.view.parseCache,
-		fset:               fileSetWithBase(reservedForParsing),
-		syntaxIndex:        make(map[PackageID]int),
-		cpulimit:           make(chan unit, runtime.GOMAXPROCS(0)),
-		syntaxPackages:     make(map[PackageID]*futurePackage),
-		importPackages:     make(map[PackageID]*futurePackage),
+		pre:            pre,
+		post:           post,
+		handles:        handles,
+		parseCache:     s.view.parseCache,
+		fset:           fileSetWithBase(reservedForParsing),
+		syntaxIndex:    make(map[PackageID]int),
+		cpulimit:       make(chan unit, runtime.GOMAXPROCS(0)),
+		syntaxPackages: make(map[PackageID]*futurePackage),
+		importPackages: make(map[PackageID]*futurePackage),
 	}
 
 	if importGraph != nil {
@@ -461,16 +494,17 @@ func (b *typeCheckBatch) getImportPackage(ctx context.Context, id PackageID) (pk
 		// type-checking was short-circuited by the pre- func.
 	}
 
-	// unsafe cannot be imported or type-checked.
-	if id == "unsafe" {
-		return types.Unsafe, nil
-	}
-
 	ph := b.handles[id]
 
-	// Do a second check for "unsafe" defensively, due to golang/go#60890.
+	// "unsafe" cannot be imported or type-checked.
+	//
+	// We check PkgPath, not id, as the structure of the ID
+	// depends on the build system (in particular,
+	// Bazel+gopackagesdriver appears to use something other than
+	// "unsafe", though we aren't sure what; even 'go list' can
+	// use "p [q.test]" for testing or if PGO is enabled.
+	// See golang/go#60890.
 	if ph.mp.PkgPath == "unsafe" {
-		bug.Reportf("encountered \"unsafe\" as %s (golang/go#60890)", id)
 		return types.Unsafe, nil
 	}
 
@@ -513,20 +547,6 @@ func (b *typeCheckBatch) handleSyntaxPackage(ctx context.Context, i int, id Pack
 		return nil, nil // skip: export data only
 	}
 
-	// Check for existing active packages.
-	//
-	// Since gopls can't depend on package identity, any instance of the
-	// requested package must be ok to return.
-	//
-	// This is an optimization to avoid redundant type-checking: following
-	// changes to an open package many LSP clients send several successive
-	// requests for package information for the modified package (semantic
-	// tokens, code lens, inlay hints, etc.)
-	if pkg := b.activePackageCache.getActivePackage(id); pkg != nil {
-		b.post(i, pkg)
-		return nil, nil // skip: not checked in this batch
-	}
-
 	// Wait for predecessors.
 	{
 		var g errgroup.Group
@@ -567,8 +587,7 @@ func (b *typeCheckBatch) handleSyntaxPackage(ctx context.Context, i int, id Pack
 	}
 
 	// Update caches.
-	b.activePackageCache.setActivePackage(id, p) // store active packages in memory
-	go storePackageResults(ctx, ph, p)           // ...and write all packages to disk
+	go storePackageResults(ctx, ph, p) // ...and write all packages to disk
 
 	b.post(i, p)
 
@@ -592,10 +611,6 @@ func storePackageResults(ctx context.Context, ph *packageHandle, p *Package) {
 		} else {
 			toCache[exportDataKind] = exportData
 		}
-	} else if p.metadata.ID != "unsafe" {
-		// golang/go#60890: we should only ever see one variant of the "unsafe"
-		// package.
-		bug.Reportf("encountered \"unsafe\" as %s (golang/go#60890)", p.metadata.ID)
 	}
 
 	for kind, data := range toCache {
@@ -608,7 +623,7 @@ func storePackageResults(ctx context.Context, ph *packageHandle, p *Package) {
 // importPackage loads the given package from its export data in p.exportData
 // (which must already be populated).
 func (b *typeCheckBatch) importPackage(ctx context.Context, mp *metadata.Package, data []byte) (*types.Package, error) {
-	ctx, done := event.Start(ctx, "cache.typeCheckBatch.importPackage", tag.Package.Of(string(mp.ID)))
+	ctx, done := event.Start(ctx, "cache.typeCheckBatch.importPackage", label.Package.Of(string(mp.ID)))
 	defer done()
 
 	impMap := b.importMap(mp.ID)
@@ -674,7 +689,7 @@ func (b *typeCheckBatch) importPackage(ctx context.Context, mp *metadata.Package
 // checkPackageForImport type checks, but skips function bodies and does not
 // record syntax information.
 func (b *typeCheckBatch) checkPackageForImport(ctx context.Context, ph *packageHandle) (*types.Package, error) {
-	ctx, done := event.Start(ctx, "cache.typeCheckBatch.checkPackageForImport", tag.Package.Of(string(ph.mp.ID)))
+	ctx, done := event.Start(ctx, "cache.typeCheckBatch.checkPackageForImport", label.Package.Of(string(ph.mp.ID)))
 	defer done()
 
 	onError := func(e error) {
@@ -1347,9 +1362,9 @@ type typeCheckInputs struct {
 	// TODO(rfindley): consider storing less data in gobDiagnostics, and
 	// interpreting each diagnostic in the context of a fixed set of options.
 	// Then these fields need not be part of the type checking inputs.
-	relatedInformation bool
-	linkTarget         string
-	moduleMode         bool
+	supportsRelatedInformation bool
+	linkTarget                 string
+	viewType                   ViewType
 }
 
 func (s *Snapshot) typeCheckInputs(ctx context.Context, mp *metadata.Package) (typeCheckInputs, error) {
@@ -1387,9 +1402,9 @@ func (s *Snapshot) typeCheckInputs(ctx context.Context, mp *metadata.Package) (t
 		depsByImpPath:   mp.DepsByImpPath,
 		goVersion:       goVersion,
 
-		relatedInformation: s.Options().RelatedInformationSupported,
-		linkTarget:         s.Options().LinkTarget,
-		moduleMode:         s.view.moduleMode(),
+		supportsRelatedInformation: s.Options().RelatedInformationSupported,
+		linkTarget:                 s.Options().LinkTarget,
+		viewType:                   s.view.typ,
 	}, nil
 }
 
@@ -1446,9 +1461,9 @@ func localPackageKey(inputs typeCheckInputs) file.Hash {
 	maxAlign := inputs.sizes.Alignof(types.NewPointer(types.Typ[types.Int64]))
 	fmt.Fprintf(hasher, "sizes: %d %d\n", wordSize, maxAlign)
 
-	fmt.Fprintf(hasher, "relatedInformation: %t\n", inputs.relatedInformation)
+	fmt.Fprintf(hasher, "relatedInformation: %t\n", inputs.supportsRelatedInformation)
 	fmt.Fprintf(hasher, "linkTarget: %s\n", inputs.linkTarget)
-	fmt.Fprintf(hasher, "moduleMode: %t\n", inputs.moduleMode)
+	fmt.Fprintf(hasher, "viewType: %d\n", inputs.viewType)
 
 	var hash [sha256.Size]byte
 	hasher.Sum(hash[:0])
@@ -1460,7 +1475,7 @@ func localPackageKey(inputs typeCheckInputs) file.Hash {
 // deps holds the future results of type-checking the direct dependencies.
 func (b *typeCheckBatch) checkPackage(ctx context.Context, ph *packageHandle) (*Package, error) {
 	inputs := ph.localInputs
-	ctx, done := event.Start(ctx, "cache.typeCheckBatch.checkPackage", tag.Package.Of(string(inputs.id)))
+	ctx, done := event.Start(ctx, "cache.typeCheckBatch.checkPackage", label.Package.Of(string(inputs.id)))
 	defer done()
 
 	pkg := &syntaxPackage{
@@ -1577,7 +1592,7 @@ func (b *typeCheckBatch) checkPackage(ctx context.Context, ph *packageHandle) (*
 	for _, e := range pkg.parseErrors {
 		diags, err := parseErrorDiagnostics(pkg, e)
 		if err != nil {
-			event.Error(ctx, "unable to compute positions for parse errors", err, tag.Package.Of(string(inputs.id)))
+			event.Error(ctx, "unable to compute positions for parse errors", err, label.Package.Of(string(inputs.id)))
 			continue
 		}
 		for _, diag := range diags {
@@ -1586,7 +1601,7 @@ func (b *typeCheckBatch) checkPackage(ctx context.Context, ph *packageHandle) (*
 		}
 	}
 
-	diags := typeErrorsToDiagnostics(pkg, pkg.typeErrors, inputs.linkTarget, inputs.moduleMode, inputs.relatedInformation)
+	diags := typeErrorsToDiagnostics(pkg, inputs, pkg.typeErrors)
 	for _, diag := range diags {
 		// If the file didn't parse cleanly, it is highly likely that type
 		// checking errors will be confusing or redundant. But otherwise, type
@@ -1621,9 +1636,9 @@ func (b *typeCheckBatch) typesConfig(ctx context.Context, inputs typeCheckInputs
 			depPH := b.handles[id]
 			if depPH == nil {
 				// e.g. missing metadata for dependencies in buildPackageHandle
-				return nil, missingPkgError(inputs.id, path, inputs.moduleMode)
+				return nil, missingPkgError(inputs.id, path, inputs.viewType)
 			}
-			if !metadata.IsValidImport(inputs.pkgPath, depPH.mp.PkgPath) {
+			if !metadata.IsValidImport(inputs.pkgPath, depPH.mp.PkgPath, inputs.viewType != GoPackagesDriverView) {
 				return nil, fmt.Errorf("invalid use of internal package %q", path)
 			}
 			return b.getImportPackage(ctx, id)
@@ -1654,6 +1669,10 @@ func validGoVersion(goVersion string) bool {
 		return false // malformed version string
 	}
 
+	if relVer := releaseVersion(); relVer != "" && versions.Before(versions.Lang(relVer), versions.Lang(goVersion)) {
+		return false // 'go list' is too new for go/types
+	}
+
 	// TODO(rfindley): remove once we no longer support building gopls with Go
 	// 1.20 or earlier.
 	if !slices.Contains(build.Default.ReleaseTags, "go1.21") && strings.Count(goVersion, ".") >= 2 {
@@ -1663,6 +1682,19 @@ func validGoVersion(goVersion string) bool {
 	return true
 }
 
+// releaseVersion reports the Go language version used to compile gopls, or ""
+// if it cannot be determined.
+func releaseVersion() string {
+	if len(build.Default.ReleaseTags) > 0 {
+		v := build.Default.ReleaseTags[len(build.Default.ReleaseTags)-1]
+		var dummy int
+		if _, err := fmt.Sscanf(v, "go1.%d", &dummy); err == nil {
+			return v
+		}
+	}
+	return ""
+}
+
 // depsErrors creates diagnostics for each metadata error (e.g. import cycle).
 // These may be attached to import declarations in the transitive source files
 // of pkg, or to 'requires' declarations in the package's go.mod file.
@@ -1744,7 +1776,7 @@ func depsErrors(ctx context.Context, snapshot *Snapshot, mp *metadata.Package) (
 					Message:        fmt.Sprintf("error while importing %v: %v", item, depErr.Err),
 					SuggestedFixes: goGetQuickFixes(mp.Module != nil, imp.cgf.URI, item),
 				}
-				if !bundleQuickFixes(diag) {
+				if !bundleLazyFixes(diag) {
 					bug.Reportf("failed to bundle fixes for diagnostic %q", diag.Message)
 				}
 				errors = append(errors, diag)
@@ -1787,7 +1819,7 @@ func depsErrors(ctx context.Context, snapshot *Snapshot, mp *metadata.Package) (
 				Message:        fmt.Sprintf("error while importing %v: %v", item, depErr.Err),
 				SuggestedFixes: goGetQuickFixes(true, pm.URI, item),
 			}
-			if !bundleQuickFixes(diag) {
+			if !bundleLazyFixes(diag) {
 				bug.Reportf("failed to bundle fixes for diagnostic %q", diag.Message)
 			}
 			errors = append(errors, diag)
@@ -1799,20 +1831,23 @@ func depsErrors(ctx context.Context, snapshot *Snapshot, mp *metadata.Package) (
 
 // missingPkgError returns an error message for a missing package that varies
 // based on the user's workspace mode.
-func missingPkgError(from PackageID, pkgPath string, moduleMode bool) error {
-	// TODO(rfindley): improve this error. Previous versions of this error had
-	// access to the full snapshot, and could provide more information (such as
-	// the initialization error).
-	if moduleMode {
+func missingPkgError(from PackageID, pkgPath string, viewType ViewType) error {
+	switch viewType {
+	case GoModView, GoWorkView:
 		if metadata.IsCommandLineArguments(from) {
 			return fmt.Errorf("current file is not included in a workspace module")
 		} else {
 			// Previously, we would present the initialization error here.
 			return fmt.Errorf("no required module provides package %q", pkgPath)
 		}
-	} else {
-		// Previously, we would list the directories in GOROOT and GOPATH here.
+	case AdHocView:
+		return fmt.Errorf("cannot find package %q in GOROOT", pkgPath)
+	case GoPackagesDriverView:
+		return fmt.Errorf("go/packages driver could not load %q", pkgPath)
+	case GOPATHView:
 		return fmt.Errorf("cannot find package %q in GOROOT or GOPATH", pkgPath)
+	default:
+		return fmt.Errorf("unable to load package")
 	}
 }
 
@@ -1826,12 +1861,12 @@ func missingPkgError(from PackageID, pkgPath string, moduleMode bool) error {
 // to the previous error in the errs slice (such as if they were printed in
 // sequence to a terminal).
 //
-// The linkTarget, moduleMode, and supportsRelatedInformation parameters affect
-// the construction of protocol objects (see the code for details).
-func typeErrorsToDiagnostics(pkg *syntaxPackage, errs []types.Error, linkTarget string, moduleMode, supportsRelatedInformation bool) []*Diagnostic {
+// Fields in typeCheckInputs may affect the resulting diagnostics.
+func typeErrorsToDiagnostics(pkg *syntaxPackage, inputs typeCheckInputs, errs []types.Error) []*Diagnostic {
 	var result []*Diagnostic
 
 	// batch records diagnostics for a set of related types.Errors.
+	// (related[0] is the primary error.)
 	batch := func(related []types.Error) {
 		var diags []*Diagnostic
 		for i, e := range related {
@@ -1915,9 +1950,9 @@ func typeErrorsToDiagnostics(pkg *syntaxPackage, errs []types.Error, linkTarget
 				bug.Reportf("internal error: could not compute pos to range for %v: %v", e, err)
 				continue
 			}
-			msg := related[0].Msg
+			msg := related[0].Msg // primary
 			if i > 0 {
-				if supportsRelatedInformation {
+				if inputs.supportsRelatedInformation {
 					msg += " (see details)"
 				} else {
 					msg += fmt.Sprintf(" (this error: %v)", e.Msg)
@@ -1932,16 +1967,16 @@ func typeErrorsToDiagnostics(pkg *syntaxPackage, errs []types.Error, linkTarget
 			}
 			if code != 0 {
 				diag.Code = code.String()
-				diag.CodeHref = typesCodeHref(linkTarget, code)
+				diag.CodeHref = typesCodeHref(inputs.linkTarget, code)
 			}
 			if code == typesinternal.UnusedVar || code == typesinternal.UnusedImport {
 				diag.Tags = append(diag.Tags, protocol.Unnecessary)
 			}
 			if match := importErrorRe.FindStringSubmatch(e.Msg); match != nil {
-				diag.SuggestedFixes = append(diag.SuggestedFixes, goGetQuickFixes(moduleMode, pgf.URI, match[1])...)
+				diag.SuggestedFixes = append(diag.SuggestedFixes, goGetQuickFixes(inputs.viewType.usesModules(), pgf.URI, match[1])...)
 			}
 			if match := unsupportedFeatureRe.FindStringSubmatch(e.Msg); match != nil {
-				diag.SuggestedFixes = append(diag.SuggestedFixes, editGoDirectiveQuickFix(moduleMode, pgf.URI, match[1])...)
+				diag.SuggestedFixes = append(diag.SuggestedFixes, editGoDirectiveQuickFix(inputs.viewType.usesModules(), pgf.URI, match[1])...)
 			}
 
 			// Link up related information. For the primary error, all related errors
@@ -1951,7 +1986,10 @@ func typeErrorsToDiagnostics(pkg *syntaxPackage, errs []types.Error, linkTarget
 			// This is because go/types assumes that errors are read top-down, such as
 			// in the cycle error "A refers to...". The structure of the secondary
 			// error set likely only makes sense for the primary error.
-			if i > 0 {
+			//
+			// NOTE: len(diags) == 0 if the primary diagnostic has invalid positions.
+			// See also golang/go#66731.
+			if i > 0 && len(diags) > 0 {
 				primary := diags[0]
 				primary.Related = append(primary.Related, protocol.DiagnosticRelatedInformation{
 					Location: protocol.Location{URI: diag.URI, Range: diag.Range},
