@@ -11,8 +11,8 @@ import (
 	"testing"
 
 	"golang.org/x/tools/gopls/internal/cache/parsego"
+	"golang.org/x/tools/gopls/internal/tokeninternal"
 	"golang.org/x/tools/gopls/internal/util/safetoken"
-	"golang.org/x/tools/internal/tokeninternal"
 )
 
 // TODO(golang/go#64335): we should have many more tests for fixed syntax.
@@ -32,7 +32,7 @@ func _() {
 }
 `
 
-	pgf, _ := parsego.Parse(context.Background(), token.NewFileSet(), "file://foo.go", []byte(src), parsego.ParseFull, false)
+	pgf, _ := parsego.Parse(context.Background(), token.NewFileSet(), "file://foo.go", []byte(src), parsego.Full, false)
 	fset := tokeninternal.FileSetFor(pgf.Tok)
 	ast.Inspect(pgf.File, func(n ast.Node) bool {
 		if n != nil {
