@@ -40,7 +40,6 @@ import (
 
 	"golang.org/x/tools/gopls/internal/cmd"
 	"golang.org/x/tools/gopls/internal/debug"
-	"golang.org/x/tools/gopls/internal/hooks"
 	"golang.org/x/tools/gopls/internal/protocol"
 	"golang.org/x/tools/gopls/internal/util/bug"
 	"golang.org/x/tools/gopls/internal/version"
@@ -103,6 +102,12 @@ var _ = fmt.Sprintf("%s", 123)
 package a
 import "fmt"
 var _ = fmt.Sprintf("%d", "123")
+-- c/c.go --
+package c
+var C int
+-- c/c2.go --
+package c
+var C int
 `)
 
 	// no files
@@ -128,6 +133,14 @@ var _ = fmt.Sprintf("%d", "123")
 		res.checkStdout(`a.go:.* fmt.Sprintf format %s has arg 123 of wrong type int`)
 		res.checkStdout(`b.go:.* fmt.Sprintf format %d has arg "123" of wrong type string`)
 	}
+
+	// diagnostic with related information spanning files
+	{
+		res := gopls(t, tree, "check", "./c/c2.go")
+		res.checkExit(true)
+		res.checkStdout(`c2.go:2:5-6: C redeclared in this block`)
+		res.checkStdout(`c.go:2:5-6: - other declaration of C`)
+	}
 }
 
 // TestCallHierarchy tests the 'call_hierarchy' subcommand (../call_hierarchy.go).
@@ -806,7 +819,7 @@ const c = 0
 		want := `
 /*⇒7,keyword,[]*/package /*⇒1,namespace,[]*/a
 /*⇒4,keyword,[]*/func /*⇒1,function,[definition]*/f()
-/*⇒3,keyword,[]*/var /*⇒1,variable,[definition]*/v /*⇒3,type,[defaultLibrary]*/int
+/*⇒3,keyword,[]*/var /*⇒1,variable,[definition]*/v /*⇒3,type,[defaultLibrary number]*/int
 /*⇒5,keyword,[]*/const /*⇒1,variable,[definition readonly]*/c = /*⇒1,number,[]*/0
 `[1:]
 		if got != want {
@@ -928,8 +941,8 @@ package foo
 	}
 }
 
-// TestFix tests the 'fix' subcommand (../suggested_fix.go).
-func TestFix(t *testing.T) {
+// TestCodeAction tests the 'codeaction' subcommand (../codeaction.go).
+func TestCodeAction(t *testing.T) {
 	t.Parallel()
 
 	tree := writeTree(t, `
@@ -951,29 +964,49 @@ type C struct{}
 
 	// no arguments
 	{
-		res := gopls(t, tree, "fix")
+		res := gopls(t, tree, "codeaction")
 		res.checkExit(false)
 		res.checkStderr("expects at least 1 argument")
 	}
-	// success with default kinds, {quickfix}.
-	// -a is always required because no fix is currently "preferred" (!)
+	// list code actions in file
 	{
-		res := gopls(t, tree, "fix", "-a", "a.go")
+		res := gopls(t, tree, "codeaction", "a.go")
+		res.checkExit(true)
+		res.checkStdout(`edit	"Fill in return values" \[quickfix\]`)
+		res.checkStdout(`command	"Browse documentation for package a" \[source.doc\]`)
+	}
+	// list code actions in file, filtering by title
+	{
+		res := gopls(t, tree, "codeaction", "-title=Browse.*doc", "a.go")
 		res.checkExit(true)
 		got := res.stdout
-		want := `
-package a
-type T int
-func f() (int, string) { return 0, "" }
-
-`[1:]
+		want := `command	"Browse gopls feature documentation" [gopls.doc.features]` +
+			"\n" +
+			`command	"Browse documentation for package a" [source.doc]` +
+			"\n"
 		if got != want {
-			t.Errorf("fix: got <<%s>>, want <<%s>>\nstderr:\n%s", got, want, res.stderr)
+			t.Errorf("codeaction: got <<%s>>, want <<%s>>\nstderr:\n%s", got, want, res.stderr)
+		}
+	}
+	// list code actions at position (of io.Reader)
+	{
+		res := gopls(t, tree, "codeaction", "b.go:#31")
+		res.checkExit(true)
+		res.checkStdout(`command	"Browse documentation for type io.Reader" \[source.doc]`)
+	}
+	// list quick fixes at position (of type T)
+	{
+		res := gopls(t, tree, "codeaction", "-kind=quickfix", "a.go:#15")
+		res.checkExit(true)
+		got := res.stdout
+		want := `edit	"Fill in return values" [quickfix]` + "\n"
+		if got != want {
+			t.Errorf("codeaction: got <<%s>>, want <<%s>>\nstderr:\n%s", got, want, res.stderr)
 		}
 	}
 	// success, with explicit CodeAction kind and diagnostics span.
 	{
-		res := gopls(t, tree, "fix", "-a", "b.go:#40", "quickfix")
+		res := gopls(t, tree, "codeaction", "-kind=quickfix", "-exec", "b.go:#40")
 		res.checkExit(true)
 		got := res.stdout
 		want := `
@@ -991,7 +1024,7 @@ func (c C) Read(p []byte) (n int, err error) {
 }
 `[1:]
 		if got != want {
-			t.Errorf("fix: got <<%s>>, want <<%s>>\nstderr:\n%s", got, want, res.stderr)
+			t.Errorf("codeaction: got <<%s>>, want <<%s>>\nstderr:\n%s", got, want, res.stderr)
 		}
 	}
 }
@@ -1046,7 +1079,7 @@ func goplsMain() {
 		version.VersionOverride = v
 	}
 
-	tool.Main(context.Background(), cmd.New(hooks.Options), os.Args[1:])
+	tool.Main(context.Background(), cmd.New(), os.Args[1:])
 }
 
 // writeTree extracts a txtar archive into a new directory and returns its path.
