@@ -10,9 +10,9 @@ import (
 	"context"
 	"fmt"
 
+	label1 "golang.org/x/tools/gopls/internal/label"
 	"golang.org/x/tools/internal/event"
 	"golang.org/x/tools/internal/event/label"
-	"golang.org/x/tools/internal/event/tag"
 )
 
 // Level parameterizes log severity.
@@ -29,7 +29,7 @@ const (
 
 // Log exports a log event labeled with level l.
 func (l Level) Log(ctx context.Context, msg string) {
-	event.Log(ctx, msg, tag.Level.Of(int(l)))
+	event.Log(ctx, msg, label1.Level.Of(int(l)))
 }
 
 // Logf formats and exports a log event labeled with level l.
@@ -39,5 +39,5 @@ func (l Level) Logf(ctx context.Context, format string, args ...interface{}) {
 
 // LabeledLevel extracts the labeled log l
 func LabeledLevel(lm label.Map) Level {
-	return Level(tag.Level.Get(lm))
+	return Level(label1.Level.Get(lm))
 }
