@@ -21,7 +21,6 @@ var JSON string
 // TODO(adonovan): document these data types.
 type API struct {
 	Options   map[string][]*Option
-	Commands  []*Command
 	Lenses    []*Lens
 	Analyzers []*Analyzer
 	Hints     []*Hint
@@ -54,14 +53,6 @@ type EnumValue struct {
 	Doc   string // doc comment; always starts with `Value`
 }
 
-type Command struct {
-	Command   string // e.g. "gopls.run_tests"
-	Title     string
-	Doc       string
-	ArgDoc    string
-	ResultDoc string
-}
-
 type Lens struct {
 	FileType string // e.g. "Go", "go.mod"
 	Lens     string
