@@ -17,7 +17,6 @@ package golang
 // - list promoted methods---we have type information!
 // - gather Example tests, following go/doc and pkgsite.
 // - add option for doc.AllDecls: show non-exported symbols too.
-// - abbreviate long signatures by replacing parameters 4 onwards with "...".
 // - style the <li> bullets in the index as invisible.
 // - add push notifications such as didChange -> reload.
 // - there appears to be a maximum file size beyond which the
@@ -25,9 +24,9 @@ package golang
 // - modify JS httpGET function to give a transient visual indication
 //   when clicking a source link that the editor is being navigated
 //   (in case it doesn't raise itself, like VS Code).
-// - move this into a new package, golang/pkgdoc, and then
+// - move this into a new package, golang/web, and then
 //   split out the various helpers without fear of polluting
-//   the golang package namespace.
+//   the golang package namespace?
 // - show "Deprecated" chip when appropriate.
 
 import (
@@ -43,36 +42,235 @@ import (
 	"path/filepath"
 	"strings"
 
+	"golang.org/x/tools/go/ast/astutil"
 	"golang.org/x/tools/gopls/internal/cache"
+	"golang.org/x/tools/gopls/internal/cache/parsego"
 	"golang.org/x/tools/gopls/internal/protocol"
-	"golang.org/x/tools/gopls/internal/util/astutil"
+	goplsastutil "golang.org/x/tools/gopls/internal/util/astutil"
 	"golang.org/x/tools/gopls/internal/util/bug"
 	"golang.org/x/tools/gopls/internal/util/safetoken"
 	"golang.org/x/tools/gopls/internal/util/slices"
 	"golang.org/x/tools/gopls/internal/util/typesutil"
+	"golang.org/x/tools/internal/stdlib"
 	"golang.org/x/tools/internal/typesinternal"
 )
 
-// TODO(adonovan): factor these two functions into an interface.
-type (
-	// A PkgURLFunc forms URLs of package or symbol documentation.
-	PkgURLFunc = func(path PackagePath, fragment string) protocol.URI
+// DocFragment finds the package and (optionally) symbol identified by
+// the current selection, and returns the package path and the
+// optional symbol URL fragment (e.g. "#Buffer.Len") for a symbol,
+// along with a title for the code action.
+//
+// It is called once to offer the code action, and again when the
+// command is executed. This is slightly inefficient but ensures that
+// the title and package/symbol logic are consistent in all cases.
+//
+// It returns zeroes if there is nothing to see here (e.g. reference to a builtin).
+func DocFragment(pkg *cache.Package, pgf *parsego.File, start, end token.Pos) (pkgpath PackagePath, fragment, title string) {
+	thing := thingAtPoint(pkg, pgf, start, end)
 
-	// A PosURLFunc forms URLs that cause the editor to navigate to a position.
-	PosURLFunc = func(filename string, line, col8 int) protocol.URI
-)
+	makeTitle := func(kind string, imp *types.Package, name string) string {
+		title := "Browse documentation for " + kind + " "
+		if imp != nil && imp != pkg.Types() {
+			title += imp.Name() + "."
+		}
+		return title + name
+	}
 
-// RenderPackageDoc formats the package documentation page.
+	wholePackage := func(pkg *types.Package) (PackagePath, string, string) {
+		// External test packages don't have /pkg doc pages,
+		// so instead show the doc for the package under test.
+		// (This named-based heuristic is imperfect.)
+		if forTest := strings.TrimSuffix(pkg.Path(), "_test"); forTest != pkg.Path() {
+			return PackagePath(forTest), "", makeTitle("package", nil, filepath.Base(forTest))
+		}
+
+		return PackagePath(pkg.Path()), "", makeTitle("package", nil, pkg.Name())
+	}
+
+	// Conceptually, we check cases in the order:
+	// 1. symbol
+	// 2. package
+	// 3. enclosing
+	// but the logic of cases 1 and 3 are identical, hence the odd factoring.
+
+	// Imported package?
+	if thing.pkg != nil && thing.symbol == nil {
+		return wholePackage(thing.pkg)
+	}
+
+	// Symbol?
+	var sym types.Object
+	if thing.symbol != nil {
+		sym = thing.symbol // reference to a symbol
+	} else if thing.enclosing != nil {
+		sym = thing.enclosing // selection is within a declaration of a symbol
+	}
+	if sym == nil {
+		return wholePackage(pkg.Types()) // no symbol
+	}
+
+	// Built-in (error.Error, append or unsafe).
+	// TODO(adonovan): handle builtins in /pkg viewer.
+	if sym.Pkg() == nil {
+		return "", "", "" // nothing to see here
+	}
+	pkgpath = PackagePath(sym.Pkg().Path())
+
+	// Unexported? Show enclosing type or package.
+	if !sym.Exported() {
+		// Unexported method of exported type?
+		if fn, ok := sym.(*types.Func); ok {
+			if recv := fn.Type().(*types.Signature).Recv(); recv != nil {
+				_, named := typesinternal.ReceiverNamed(recv)
+				if named != nil && named.Obj().Exported() {
+					sym = named.Obj()
+					goto below
+				}
+			}
+		}
+
+		return wholePackage(sym.Pkg())
+	below:
+	}
+
+	// Reference to symbol in external test package?
+	// Short-circuit: see comment in wholePackage.
+	if strings.HasSuffix(string(pkgpath), "_test") {
+		return wholePackage(pkg.Types())
+	}
+
+	// package-level symbol?
+	if isPackageLevel(sym) {
+		return pkgpath, sym.Name(), makeTitle(objectKind(sym), sym.Pkg(), sym.Name())
+	}
+
+	// Inv: sym is field or method, or local.
+	switch sym := sym.(type) {
+	case *types.Func: // => method
+		sig := sym.Type().(*types.Signature)
+		isPtr, named := typesinternal.ReceiverNamed(sig.Recv())
+		if named != nil {
+			if !named.Obj().Exported() {
+				return wholePackage(sym.Pkg()) // exported method of unexported type
+			}
+			name := fmt.Sprintf("(%s%s).%s",
+				strings.Repeat("*", btoi(isPtr)), // for *T
+				named.Obj().Name(),
+				sym.Name())
+			fragment := named.Obj().Name() + "." + sym.Name()
+			return pkgpath, fragment, makeTitle("method", sym.Pkg(), name)
+		}
+
+	case *types.Var:
+		if sym.IsField() {
+			// TODO(adonovan): support fields.
+			// The Var symbol doesn't include the struct
+			// type, so we need to use the logic from
+			// Hover. (This isn't important for
+			// DocFragment as fields don't have fragments,
+			// but it matters to the grand unification of
+			// Hover/Definition/DocFragment.
+		}
+	}
+
+	// Field, non-exported method, or local declaration:
+	// just show current package.
+	return wholePackage(pkg.Types())
+}
+
+// thing describes the package or symbol denoted by a selection.
+//
+// TODO(adonovan): Hover, Definition, and References all start by
+// identifying the selected object. Let's achieve a better factoring
+// of the common parts using this structure, including uniform
+// treatment of doc links, linkname, and suchlike.
+type thing struct {
+	// At most one of these fields is set.
+	// (The 'enclosing' field is a fallback for when neither
+	// of the first two is set.)
+	symbol    types.Object   // referenced symbol
+	pkg       *types.Package // referenced package
+	enclosing types.Object   // package-level symbol or method decl enclosing selection
+}
+
+func thingAtPoint(pkg *cache.Package, pgf *parsego.File, start, end token.Pos) thing {
+	path, _ := astutil.PathEnclosingInterval(pgf.File, start, end)
+
+	// In an import spec?
+	if len(path) >= 3 { // [...ImportSpec GenDecl File]
+		if spec, ok := path[len(path)-3].(*ast.ImportSpec); ok {
+			if pkgname, ok := typesutil.ImportedPkgName(pkg.TypesInfo(), spec); ok {
+				return thing{pkg: pkgname.Imported()}
+			}
+		}
+	}
+
+	// Definition or reference to symbol?
+	var obj types.Object
+	if id, ok := path[0].(*ast.Ident); ok {
+		obj = pkg.TypesInfo().ObjectOf(id)
+
+		// Treat use to PkgName like ImportSpec.
+		if pkgname, ok := obj.(*types.PkgName); ok {
+			return thing{pkg: pkgname.Imported()}
+		}
+
+	} else if sel, ok := path[0].(*ast.SelectorExpr); ok {
+		// e.g. selection is "fmt.Println" or just a portion ("mt.Prin")
+		obj = pkg.TypesInfo().Uses[sel.Sel]
+	}
+	if obj != nil {
+		return thing{symbol: obj}
+	}
+
+	// Find enclosing declaration.
+	if n := len(path); n > 1 {
+		switch decl := path[n-2].(type) {
+		case *ast.FuncDecl:
+			// method?
+			if fn := pkg.TypesInfo().Defs[decl.Name]; fn != nil {
+				return thing{enclosing: fn}
+			}
+
+		case *ast.GenDecl:
+			// path=[... Spec? GenDecl File]
+			for _, spec := range decl.Specs {
+				if n > 2 && spec == path[n-3] {
+					var name *ast.Ident
+					switch spec := spec.(type) {
+					case *ast.ValueSpec:
+						// var, const: use first name
+						name = spec.Names[0]
+					case *ast.TypeSpec:
+						name = spec.Name
+					}
+					if name != nil {
+						return thing{enclosing: pkg.TypesInfo().Defs[name]}
+					}
+					break
+				}
+			}
+		}
+	}
+
+	return thing{} // nothing to see here
+}
+
+// Web is an abstraction of gopls' web server.
+type Web interface {
+	// PkgURL forms URLs of package or symbol documentation.
+	PkgURL(viewID string, path PackagePath, fragment string) protocol.URI
+
+	// SrcURL forms URLs that cause the editor to open a file at a specific position.
+	SrcURL(filename string, line, col8 int) protocol.URI
+}
+
+// PackageDocHTML formats the package documentation page.
 //
 // The posURL function returns a URL that when visited, has the side
 // effect of causing gopls to direct the client editor to navigate to
 // the specified file/line/column position, in UTF-8 coordinates.
-//
-// The pkgURL function returns a URL for the documentation of the
-// specified package and symbol.
-//
-// TODO(adonovan): "Render" is a client-side verb; rename to PackageDocHTML.
-func RenderPackageDoc(pkg *cache.Package, posURL PosURLFunc, pkgURL PkgURLFunc) ([]byte, error) {
+func PackageDocHTML(viewID string, pkg *cache.Package, web Web) ([]byte, error) {
 	// We can't use doc.NewFromFiles (even with doc.PreserveAST
 	// mode) as it calls ast.NewPackage which assumes that each
 	// ast.File has an ast.Scope and resolves identifiers to
@@ -145,7 +343,7 @@ func RenderPackageDoc(pkg *cache.Package, posURL PosURLFunc, pkgURL PkgURLFunc)
 			if link.Recv != "" {
 				fragment = link.Recv + "." + link.Name
 			}
-			return pkgURL(path, fragment)
+			return web.PkgURL(viewID, path, fragment)
 		}
 		parser := docpkg.Parser()
 		parser.LookupPackage = func(name string) (importPath string, ok bool) {
@@ -204,39 +402,46 @@ func RenderPackageDoc(pkg *cache.Package, posURL PosURLFunc, pkgURL PkgURLFunc)
 <html>
 <head>
   <meta charset="UTF-8">
-  <style>` + pkgDocStyle + `</style>
   <title>` + title + `</title>
-  <script type='text/javascript'>
-// httpGET requests a URL for its effects only.
-function httpGET(url) {
-	var xhttp = new XMLHttpRequest();
-	xhttp.open("GET", url, true);
-	xhttp.send();
-	return false; // disable usual <a href=...> behavior
+  <link rel="stylesheet" href="/assets/common.css">
+  <script src="/assets/common.js"></script>
+  <style>
+.lit { color: darkgreen; }
+
+header {
+  position: sticky;
+  top: 0;
+  left: 0;
+  width: 100%;
+  padding: 0.3em;
 }
 
-window.onload = () => {
+.Documentation-sinceVersion {
+  font-weight: normal;
+  color: #808080;
+  float: right;
+}
+
+#pkgsite { height: 1.5em; }
+
+#hdr-Selector {
+  margin-right: 0.3em;
+  float: right;
+  min-width: 25em;
+  padding: 0.3em;
+}
+  </style>
+  <script type='text/javascript'>
+window.addEventListener('load', function() {
 	// Hook up the navigation selector.
 	document.getElementById('hdr-Selector').onchange = (e) => {
 		window.location.href = e.target.value;
 	};
-};
-
-// Start a GET /hang request. If it ever completes, the server
-// has disconnected. Show a banner in that case.
-{
-	var x = new XMLHttpRequest();
-	x.open("GET", "/hang", true);
-	x.onloadend = () => {
-		document.getElementById("disconnected").style.display = 'block';
-	};
-	x.send();
-};
+});
   </script>
 </head>
 <body>
 <header>
-<div id='disconnected'>Gopls server has terminated. Page is inactive.</div>
 <select id='hdr-Selector'>
 <optgroup label="Documentation">
   <option label="Overview" value="#hdr-Overview"/>
@@ -321,26 +526,6 @@ window.onload = () => {
 
 	// -- main element --
 
-	// sourceLink returns HTML for a link to open a file in the client editor.
-	sourceLink := func(text, url string) string {
-		// The /open URL returns nothing but has the side effect
-		// of causing the LSP client to open the requested file.
-		// So we use onclick to prevent the browser from navigating.
-		// We keep the href attribute as it causes the <a> to render
-		// as a link: blue, underlined, with URL hover information.
-		return fmt.Sprintf(`<a href="%[1]s" onclick='return httpGET("%[1]s")'>%[2]s</a>`,
-			escape(url), escape(text))
-	}
-
-	// objHTML returns HTML for obj.Name(), possibly as a link.
-	objHTML := func(obj types.Object) string {
-		text := obj.Name()
-		if posn := safetoken.StartPosition(pkg.FileSet(), obj.Pos()); posn.IsValid() {
-			return sourceLink(text, posURL(posn.Filename, posn.Line, posn.Column))
-		}
-		return text
-	}
-
 	// nodeHTML returns HTML markup for a syntax tree.
 	// It replaces referring identifiers with links,
 	// and adds style spans for strings and comments.
@@ -352,7 +537,7 @@ window.onload = () => {
 				// imported package name?
 				if pkgname, ok := obj.(*types.PkgName); ok {
 					// TODO(adonovan): do this for Defs of PkgName too.
-					return pkgURL(PackagePath(pkgname.Imported().Path()), "")
+					return web.PkgURL(viewID, PackagePath(pkgname.Imported().Path()), "")
 				}
 
 				// package-level symbol?
@@ -360,7 +545,7 @@ window.onload = () => {
 					if obj.Pkg() == pkg.Types() {
 						return "#" + obj.Name() // intra-package ref
 					} else {
-						return pkgURL(PackagePath(obj.Pkg().Path()), obj.Name())
+						return web.PkgURL(viewID, PackagePath(obj.Pkg().Path()), obj.Name())
 					}
 				}
 
@@ -371,7 +556,7 @@ window.onload = () => {
 						_, named := typesinternal.ReceiverNamed(sig.Recv())
 						if named != nil {
 							fragment := named.Obj().Name() + "." + fn.Name()
-							return pkgURL(PackagePath(fn.Pkg().Path()), fragment)
+							return web.PkgURL(viewID, PackagePath(fn.Pkg().Path()), fragment)
 						}
 					}
 					return ""
@@ -400,7 +585,7 @@ window.onload = () => {
 		// appear as separate decls. We should too.
 		var buf bytes.Buffer
 		for _, file := range pkg.CompiledGoFiles() {
-			if astutil.NodeContains(file.File, n.Pos()) {
+			if goplsastutil.NodeContains(file.File, n.Pos()) {
 				pos := n.Pos()
 
 				// emit emits source in the interval [pos:to] and updates pos.
@@ -461,13 +646,7 @@ window.onload = () => {
 	//   since it is unreachable for the methods in go/doc.
 	// - elides parameters after the first three: f(a, b, c, ...).
 	fnString := func(fn *types.Func) string {
-		// pkgRelative qualifies types by package name alone
-		pkgRelative := func(other *types.Package) string {
-			if pkg.Types() == other {
-				return "" // same package; unqualified
-			}
-			return other.Name()
-		}
+		pkgRelative := typesinternal.NameRelativeTo(pkg.Types())
 
 		sig := fn.Type().(*types.Signature)
 
@@ -610,6 +789,15 @@ window.onload = () => {
 		values(docpkg.Vars)
 	}
 
+	// addedInHTML returns an HTML division containing the Go release version at
+	// which this obj became available.
+	addedInHTML := func(obj types.Object) string {
+		if sym := StdSymbolOf(obj); sym != nil && sym.Version != stdlib.Version(0) {
+			return fmt.Sprintf("<span class='Documentation-sinceVersion'>added in %v</span>", sym.Version)
+		}
+		return ""
+	}
+
 	// package-level functions
 	fmt.Fprintf(&buf, "<h2 id='hdr-Functions'>Functions</h2>\n")
 	// funcs emits a list of package-level functions,
@@ -617,8 +805,9 @@ window.onload = () => {
 	funcs := func(funcs []*doc.Func) {
 		for _, docfn := range funcs {
 			obj := scope.Lookup(docfn.Name).(*types.Func)
-			fmt.Fprintf(&buf, "<h3 id='%s'>func %s</h3>\n",
-				docfn.Name, objHTML(obj))
+
+			fmt.Fprintf(&buf, "<h3 id='%s'>func %s %s</h3>\n",
+				docfn.Name, objHTML(pkg.FileSet(), web, obj), addedInHTML(obj))
 
 			// decl: func F(params) results
 			fmt.Fprintf(&buf, "<pre class='code'>%s</pre>\n",
@@ -636,7 +825,8 @@ window.onload = () => {
 		tname := scope.Lookup(doctype.Name).(*types.TypeName)
 
 		// title and source link
-		fmt.Fprintf(&buf, "<h3 id='%s'>type %s</a></h3>\n", doctype.Name, objHTML(tname))
+		fmt.Fprintf(&buf, "<h3 id='%s'>type %s %s</h3>\n",
+			doctype.Name, objHTML(pkg.FileSet(), web, tname), addedInHTML(tname))
 
 		// declaration
 		// TODO(adonovan): excise non-exported struct fields somehow.
@@ -655,9 +845,10 @@ window.onload = () => {
 		// methods on T
 		for _, docmethod := range doctype.Methods {
 			method, _, _ := types.LookupFieldOrMethod(tname.Type(), true, tname.Pkg(), docmethod.Name)
-			fmt.Fprintf(&buf, "<h4 id='%s.%s'>func (%s) %s</h4>\n",
+			fmt.Fprintf(&buf, "<h4 id='%s.%s'>func (%s) %s %s</h4>\n",
 				doctype.Name, docmethod.Name,
-				doctype.Name, objHTML(method))
+				docmethod.Orig, // T or *T
+				objHTML(pkg.FileSet(), web, method), addedInHTML(method))
 
 			// decl: func (x T) M(params) results
 			fmt.Fprintf(&buf, "<pre class='code'>%s</pre>\n",
@@ -673,7 +864,7 @@ window.onload = () => {
 	fmt.Fprintf(&buf, "<h2 id='hdr-SourceFiles'>Source files</h2>\n")
 	for _, filename := range docpkg.Filenames {
 		fmt.Fprintf(&buf, "<div class='comment'>%s</div>\n",
-			sourceLink(filepath.Base(filename), posURL(filename, 1, 1)))
+			sourceLink(filepath.Base(filename), web.SrcURL(filename, 1, 1)))
 	}
 
 	fmt.Fprintf(&buf, "</main>\n")
@@ -698,135 +889,3 @@ func typesSeqToSlice[T any](seq typesSeq[T]) []T {
 	}
 	return slice
 }
-
-// (partly taken from pkgsite's typography.css)
-const pkgDocStyle = `
-body {
-  font-family: Helvetica, Arial, sans-serif;
-  font-size: 1rem;
-  line-height: normal;
-}
-
-h1 {
-  font-size: 1.5rem;
-}
-
-h2 {
-  font-size: 1.375rem;
-}
-
-h3 {
-  font-size: 1.25rem;
-}
-
-h4 {
-  font-size: 1.125rem;
-}
-
-h5 {
-  font-size: 1rem;
-}
-
-h6 {
-  font-size: 0.875rem;
-}
-
-h1,
-h2,
-h3,
-h4 {
-  font-weight: 600;
-  line-height: 1.25em;
-  word-break: break-word;
-}
-
-h5,
-h6 {
-  font-weight: 500;
-  line-height: 1.3em;
-  word-break: break-word;
-}
-
-p {
-  font-size: 1rem;
-  line-height: 1.5rem;
-  max-width: 60rem;
-}
-
-strong {
-  font-weight: 600;
-}
-
-code,
-pre,
-textarea.code {
-  font-family: Consolas, 'Liberation Mono', Menlo, monospace;
-  font-size: 0.875rem;
-  line-height: 1.5em;
-}
-
-pre,
-textarea.code {
-  background-color: #eee;
-  border: 3px;
-  border-radius: 3px
-  color: black;
-  overflow-x: auto;
-  padding: 0.625rem;
-  tab-size: 4;
-  white-space: pre;
-}
-
-button,
-input,
-select,
-textarea {
-  font: inherit;
-}
-
-a,
-a:link,
-a:visited {
-  color: rgb(0, 125, 156);
-  text-decoration: none;
-}
-
-a:hover,
-a:focus {
-  color: rgb(0, 125, 156);
-  text-decoration: underline;
-}
-
-a:hover > * {
-  text-decoration: underline;
-}
-
-.lit { color: darkgreen; }
-
-#pkgsite { height: 1.5em; }
-
-header {
-  position: sticky;
-  top: 0;
-  left: 0;
-  width: 100%;
-  padding: 0.3em;
-}
-
-#hdr-Selector {
-  margin-right: 0.3em;
-  float: right;
-  min-width: 25em;
-  padding: 0.3em;
-}
-
-#disconnected {
-  position: fixed;
-  top: 1em;
-  left: 1em;
-  display: none; /* initially */
-  background-color: white;
-  border: thick solid red;
-  padding: 2em;
-}
-`
