@@ -34,13 +34,13 @@ import (
 	"golang.org/x/tools/gopls/internal/protocol"
 	"golang.org/x/tools/gopls/internal/protocol/command"
 	"golang.org/x/tools/gopls/internal/settings"
+	"golang.org/x/tools/gopls/internal/tokeninternal"
 	"golang.org/x/tools/gopls/internal/util/bug"
 	"golang.org/x/tools/gopls/internal/vulncheck"
 	"golang.org/x/tools/gopls/internal/vulncheck/scan"
 	"golang.org/x/tools/internal/diff"
 	"golang.org/x/tools/internal/event"
 	"golang.org/x/tools/internal/gocommand"
-	"golang.org/x/tools/internal/tokeninternal"
 	"golang.org/x/tools/internal/xcontext"
 )
 
@@ -71,6 +71,159 @@ type commandHandler struct {
 	params *protocol.ExecuteCommandParams
 }
 
+func (h *commandHandler) Modules(ctx context.Context, args command.ModulesArgs) (command.ModulesResult, error) {
+	// keepModule filters modules based on the command args
+	keepModule := func(goMod protocol.DocumentURI) bool {
+		// Does the directory enclose the view's go.mod file?
+		if !args.Dir.Encloses(goMod) {
+			return false
+		}
+
+		// Calculate the relative path
+		rel, err := filepath.Rel(args.Dir.Path(), goMod.Path())
+		if err != nil {
+			return false // "can't happen" (see prior Encloses check)
+		}
+
+		assert(filepath.Base(goMod.Path()) == "go.mod", fmt.Sprintf("invalid go.mod path: want go.mod, got %q", goMod.Path()))
+
+		// Invariant: rel is a relative path without "../" segments and the last
+		// segment is "go.mod"
+		nparts := strings.Count(rel, string(filepath.Separator))
+		return args.MaxDepth < 0 || nparts <= args.MaxDepth
+	}
+
+	// Views may include:
+	//   - go.work views containing one or more modules each;
+	//   - go.mod views containing a single module each;
+	//   - GOPATH and/or ad hoc views containing no modules.
+	//
+	// Retrieving a view via the request path would only work for a
+	// non-recursive query for a go.mod view, and even in that case
+	// [Session.SnapshotOf] doesn't work on directories. Thus we check every
+	// view.
+	var result command.ModulesResult
+	seen := map[protocol.DocumentURI]bool{}
+	for _, v := range h.s.session.Views() {
+		s, release, err := v.Snapshot()
+		if err != nil {
+			return command.ModulesResult{}, err
+		}
+		defer release()
+
+		for _, modFile := range v.ModFiles() {
+			if !keepModule(modFile) {
+				continue
+			}
+
+			// Deduplicate
+			if seen[modFile] {
+				continue
+			}
+			seen[modFile] = true
+
+			fh, err := s.ReadFile(ctx, modFile)
+			if err != nil {
+				return command.ModulesResult{}, err
+			}
+			mod, err := s.ParseMod(ctx, fh)
+			if err != nil {
+				return command.ModulesResult{}, err
+			}
+			if mod.File.Module == nil {
+				continue // syntax contains errors
+			}
+			result.Modules = append(result.Modules, command.Module{
+				Path:    mod.File.Module.Mod.Path,
+				Version: mod.File.Module.Mod.Version,
+				GoMod:   mod.URI,
+			})
+		}
+	}
+	return result, nil
+}
+
+func (h *commandHandler) Packages(ctx context.Context, args command.PackagesArgs) (command.PackagesResult, error) {
+	wantTests := args.Mode&command.NeedTests != 0
+	result := command.PackagesResult{
+		Module: make(map[string]command.Module),
+	}
+
+	keepPackage := func(pkg *metadata.Package) bool {
+		for _, file := range pkg.GoFiles {
+			for _, arg := range args.Files {
+				if file == arg || file.Dir() == arg {
+					return true
+				}
+				if args.Recursive && arg.Encloses(file) {
+					return true
+				}
+			}
+		}
+		return false
+	}
+
+	buildPackage := func(snapshot *cache.Snapshot, meta *metadata.Package) (command.Package, command.Module) {
+		if wantTests {
+			// These will be used in the next CL to query tests
+			_, _ = ctx, snapshot
+			panic("unimplemented")
+		}
+
+		pkg := command.Package{
+			Path: string(meta.PkgPath),
+		}
+		if meta.Module == nil {
+			return pkg, command.Module{}
+		}
+
+		mod := command.Module{
+			Path:    meta.Module.Path,
+			Version: meta.Module.Version,
+			GoMod:   protocol.URIFromPath(meta.Module.GoMod),
+		}
+		pkg.ModulePath = mod.Path
+		return pkg, mod
+	}
+
+	err := h.run(ctx, commandConfig{
+		progress: "Packages",
+	}, func(ctx context.Context, _ commandDeps) error {
+		for _, view := range h.s.session.Views() {
+			snapshot, release, err := view.Snapshot()
+			if err != nil {
+				return err
+			}
+			defer release()
+
+			metas, err := snapshot.WorkspaceMetadata(ctx)
+			if err != nil {
+				return err
+			}
+
+			for _, meta := range metas {
+				if meta.IsIntermediateTestVariant() {
+					continue
+				}
+				if !keepPackage(meta) {
+					continue
+				}
+
+				pkg, mod := buildPackage(snapshot, meta)
+				result.Packages = append(result.Packages, pkg)
+
+				// Overwriting is ok
+				if mod.Path != "" {
+					result.Module[mod.Path] = mod
+				}
+			}
+		}
+
+		return nil
+	})
+	return result, err
+}
+
 func (h *commandHandler) MaybePromptForTelemetry(ctx context.Context) error {
 	go h.s.maybePromptForTelemetry(ctx, true)
 	return nil
@@ -1454,6 +1607,11 @@ func (c *commandHandler) Assembly(ctx context.Context, viewID, packageID, symbol
 	return nil
 }
 
+func (c *commandHandler) ClientOpenURL(ctx context.Context, url string) error {
+	openClientBrowser(ctx, c.s.client, url)
+	return nil
+}
+
 func (c *commandHandler) ScanImports(ctx context.Context) error {
 	for _, v := range c.s.session.Views() {
 		v.ScanImports()
