@@ -10,9 +10,9 @@ package server
 import (
 	"context"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"go/build"
-	"log"
 	"os"
 	"path"
 	"path/filepath"
@@ -23,6 +23,7 @@ import (
 	"golang.org/x/telemetry/counter"
 	"golang.org/x/tools/gopls/internal/cache"
 	"golang.org/x/tools/gopls/internal/debug"
+	debuglog "golang.org/x/tools/gopls/internal/debug/log"
 	"golang.org/x/tools/gopls/internal/file"
 	"golang.org/x/tools/gopls/internal/protocol"
 	"golang.org/x/tools/gopls/internal/settings"
@@ -68,13 +69,10 @@ func (s *server) Initialize(ctx context.Context, params *protocol.ParamInitializ
 	s.progress.SetSupportsWorkDoneProgress(params.Capabilities.Window.WorkDoneProgress)
 
 	options := s.Options().Clone()
-	// TODO(rfindley): remove the error return from handleOptionResults, and
-	// eliminate this defer.
+	// TODO(rfindley): eliminate this defer.
 	defer func() { s.SetOptions(options) }()
 
-	if err := s.handleOptionResults(ctx, settings.SetOptions(options, params.InitializationOptions)); err != nil {
-		return nil, err
-	}
+	s.handleOptionErrors(ctx, options.Set(params.InitializationOptions))
 	options.ForClientCapabilities(params.ClientInfo, params.Capabilities)
 
 	if options.ShowBugReports {
@@ -84,11 +82,7 @@ func (s *server) Initialize(ctx context.Context, params *protocol.ParamInitializ
 				Type:    protocol.Error,
 				Message: fmt.Sprintf("A bug occurred on the server: %s\nLocation:%s", b.Description, b.Key),
 			}
-			go func() {
-				if err := s.eventuallyShowMessage(context.Background(), msg); err != nil {
-					log.Printf("error showing bug: %v", err)
-				}
-			}()
+			go s.eventuallyShowMessage(context.Background(), msg)
 		})
 	}
 
@@ -101,15 +95,7 @@ func (s *server) Initialize(ctx context.Context, params *protocol.ParamInitializ
 			}}
 		}
 	}
-	for _, folder := range folders {
-		if folder.URI == "" {
-			return nil, fmt.Errorf("empty WorkspaceFolder.URI")
-		}
-		if _, err := protocol.ParseDocumentURI(folder.URI); err != nil {
-			return nil, fmt.Errorf("invalid WorkspaceFolder.URI: %v", err)
-		}
-		s.pendingFolders = append(s.pendingFolders, folder)
-	}
+	s.pendingFolders = append(s.pendingFolders, folders...)
 
 	var codeActionProvider interface{} = true
 	if ca := params.Capabilities.TextDocument.CodeAction; len(ca.CodeActionLiteralSupport.CodeActionKind.ValueSet) > 0 {
@@ -284,11 +270,28 @@ func go1Point() int {
 // addFolders adds the specified list of "folders" (that's Windows for
 // directories) to the session. It does not return an error, though it
 // may report an error to the client over LSP if one or more folders
-// had problems.
+// had problems, for example, folders with unsupported file system.
 func (s *server) addFolders(ctx context.Context, folders []protocol.WorkspaceFolder) {
 	originalViews := len(s.session.Views())
 	viewErrors := make(map[protocol.URI]error)
 
+	// Skip non-'file' scheme, or invalid workspace folders,
+	// and log them form error reports.
+	// VS Code's file system API
+	// (https://code.visualstudio.com/api/references/vscode-api#FileSystem)
+	// allows extension to define their own schemes and register
+	// them with the workspace. We've seen gitlens://, decompileFs://, etc
+	// but the list can grow over time.
+	var filtered []protocol.WorkspaceFolder
+	for _, f := range folders {
+		if _, err := protocol.ParseDocumentURI(f.URI); err != nil {
+			debuglog.Warning.Logf(ctx, "skip adding virtual folder %q - invalid folder URI: %v", f.Name, err)
+			continue
+		}
+		filtered = append(filtered, f)
+	}
+	folders = filtered
+
 	var ndiagnose sync.WaitGroup // number of unfinished diagnose calls
 	if s.Options().VerboseWorkDoneProgress {
 		work := s.progress.Start(ctx, DiagnosticWorkTitle(FromInitialWorkspaceLoad), "Calculating diagnostics for initial workspace load...", nil, nil)
@@ -332,7 +335,7 @@ func (s *server) addFolders(ctx context.Context, folders []protocol.WorkspaceFol
 		// Diagnose the newly created view asynchronously.
 		ndiagnose.Add(1)
 		go func() {
-			s.diagnoseSnapshot(snapshot, nil, 0)
+			s.diagnoseSnapshot(snapshot.BackgroundContext(), snapshot, nil, 0)
 			<-initialized
 			release()
 			ndiagnose.Done()
@@ -460,38 +463,29 @@ func (s *server) SetOptions(opts *settings.Options) {
 	s.options = opts
 }
 
-func (s *server) newFolder(ctx context.Context, folder protocol.DocumentURI, name string) (*cache.Folder, error) {
-	opts := s.Options()
-	if opts.ConfigurationSupported {
-		scope := string(folder)
-		configs, err := s.client.Configuration(ctx, &protocol.ParamConfiguration{
-			Items: []protocol.ConfigurationItem{{
-				ScopeURI: &scope,
-				Section:  "gopls",
-			}},
-		},
-		)
-		if err != nil {
-			return nil, fmt.Errorf("failed to get workspace configuration from client (%s): %v", folder, err)
-		}
-
-		opts = opts.Clone()
-		for _, config := range configs {
-			if err := s.handleOptionResults(ctx, settings.SetOptions(opts, config)); err != nil {
-				return nil, err
-			}
-		}
-	}
-
+func (s *server) newFolder(ctx context.Context, folder protocol.DocumentURI, name string, opts *settings.Options) (*cache.Folder, error) {
 	env, err := cache.FetchGoEnv(ctx, folder, opts)
 	if err != nil {
 		return nil, err
 	}
+
+	// Increment folder counters.
+	switch {
+	case env.GOTOOLCHAIN == "auto" || strings.Contains(env.GOTOOLCHAIN, "+auto"):
+		counter.New("gopls/gotoolchain:auto").Inc()
+	case env.GOTOOLCHAIN == "path" || strings.Contains(env.GOTOOLCHAIN, "+path"):
+		counter.New("gopls/gotoolchain:path").Inc()
+	case env.GOTOOLCHAIN == "local": // local+auto and local+path handled above
+		counter.New("gopls/gotoolchain:local").Inc()
+	default:
+		counter.New("gopls/gotoolchain:other").Inc()
+	}
+
 	return &cache.Folder{
 		Dir:     folder,
 		Name:    name,
 		Options: opts,
-		Env:     env,
+		Env:     *env,
 	}, nil
 }
 
@@ -522,33 +516,30 @@ func (s *server) fetchFolderOptions(ctx context.Context, folder protocol.Documen
 
 	opts = opts.Clone()
 	for _, config := range configs {
-		if err := s.handleOptionResults(ctx, settings.SetOptions(opts, config)); err != nil {
-			return nil, err
-		}
+		s.handleOptionErrors(ctx, opts.Set(config))
 	}
 	return opts, nil
 }
 
-func (s *server) eventuallyShowMessage(ctx context.Context, msg *protocol.ShowMessageParams) error {
+func (s *server) eventuallyShowMessage(ctx context.Context, msg *protocol.ShowMessageParams) {
 	s.stateMu.Lock()
 	defer s.stateMu.Unlock()
 	if s.state == serverInitialized {
-		return s.client.ShowMessage(ctx, msg)
+		_ = s.client.ShowMessage(ctx, msg) // ignore error
 	}
 	s.notifications = append(s.notifications, msg)
-	return nil
 }
 
-func (s *server) handleOptionResults(ctx context.Context, results settings.OptionResults) error {
-	var warnings, errors []string
-	for _, result := range results {
-		switch result.Error.(type) {
-		case nil:
-			// nothing to do
-		case *settings.SoftError:
-			warnings = append(warnings, result.Error.Error())
-		default:
-			errors = append(errors, result.Error.Error())
+func (s *server) handleOptionErrors(ctx context.Context, optionErrors []error) {
+	var warnings, errs []string
+	for _, err := range optionErrors {
+		if err == nil {
+			panic("nil error passed to handleOptionErrors")
+		}
+		if errors.Is(err, new(settings.SoftError)) {
+			warnings = append(warnings, err.Error())
+		} else {
+			errs = append(errs, err.Error())
 		}
 	}
 
@@ -560,10 +551,10 @@ func (s *server) handleOptionResults(ctx context.Context, results settings.Optio
 	// individual viewsettings.
 	var msgs []string
 	msgType := protocol.Warning
-	if len(errors) > 0 {
+	if len(errs) > 0 {
 		msgType = protocol.Error
-		sort.Strings(errors)
-		msgs = append(msgs, errors...)
+		sort.Strings(errs)
+		msgs = append(msgs, errs...)
 	}
 	if len(warnings) > 0 {
 		sort.Strings(warnings)
@@ -577,10 +568,8 @@ func (s *server) handleOptionResults(ctx context.Context, results settings.Optio
 			Type:    msgType,
 			Message: combined,
 		}
-		return s.eventuallyShowMessage(ctx, params)
+		s.eventuallyShowMessage(ctx, params)
 	}
-
-	return nil
 }
 
 // fileOf returns the file for a given URI and its snapshot.
