@@ -2,12 +2,16 @@
 // Use of this source code is governed by a BSD-style
 // license that can be found in the LICENSE file.
 
-package settings
+package settings_test
 
 import (
 	"reflect"
 	"testing"
 	"time"
+
+	"github.com/google/go-cmp/cmp"
+	"golang.org/x/tools/gopls/internal/clonetest"
+	. "golang.org/x/tools/gopls/internal/settings"
 )
 
 func TestDefaultsEquivalence(t *testing.T) {
@@ -18,7 +22,7 @@ func TestDefaultsEquivalence(t *testing.T) {
 	}
 }
 
-func TestSetOption(t *testing.T) {
+func TestOptions_Set(t *testing.T) {
 	type testCase struct {
 		name      string
 		value     any
@@ -206,10 +210,18 @@ func TestSetOption(t *testing.T) {
 
 	for _, test := range tests {
 		var opts Options
-		result := opts.set(test.name, test.value, map[string]struct{}{})
-		if (result.Error != nil) != test.wantError {
-			t.Fatalf("Options.set(%q, %v): result.Error = %v, want error: %t", test.name, test.value, result.Error, test.wantError)
+		err := opts.Set(map[string]any{test.name: test.value})
+		if err != nil {
+			if !test.wantError {
+				t.Errorf("Options.set(%q, %v) failed: %v",
+					test.name, test.value, err)
+			}
+			continue
+		} else if test.wantError {
+			t.Fatalf("Options.set(%q, %v) succeeded unexpectedly",
+				test.name, test.value)
 		}
+
 		// TODO: this could be made much better using cmp.Diff, if that becomes
 		// available in this module.
 		if !test.check(opts) {
@@ -217,3 +229,23 @@ func TestSetOption(t *testing.T) {
 		}
 	}
 }
+
+func TestOptions_Clone(t *testing.T) {
+	// Test that the Options.Clone actually performs a deep clone of the Options
+	// struct.
+
+	golden := clonetest.NonZero[*Options]()
+	opts := clonetest.NonZero[*Options]()
+	opts2 := opts.Clone()
+
+	// The clone should be equivalent to the original.
+	if diff := cmp.Diff(golden, opts2); diff != "" {
+		t.Errorf("Clone() does not match original (-want +got):\n%s", diff)
+	}
+
+	// Mutating the clone should not mutate the original.
+	clonetest.ZeroOut(opts2)
+	if diff := cmp.Diff(golden, opts); diff != "" {
+		t.Errorf("Mutating clone mutated the original (-want +got):\n%s", diff)
+	}
+}
