@@ -6,6 +6,8 @@ package integration
 
 import (
 	"encoding/json"
+	"errors"
+	"os"
 	"path"
 
 	"golang.org/x/tools/gopls/internal/protocol"
@@ -114,16 +116,24 @@ func (e *Env) SetBufferContent(name string, content string) {
 	}
 }
 
-// ReadFile returns the file content for name that applies to the current
-// editing session: if the file is open, it returns its buffer content,
-// otherwise it returns on disk content.
+// FileContent returns the file content for name that applies to the current
+// editing session: it returns the buffer content for an open file, the
+// on-disk content for an unopened file, or "" for a non-existent file.
 func (e *Env) FileContent(name string) string {
 	e.T.Helper()
 	text, ok := e.Editor.BufferText(name)
 	if ok {
 		return text
 	}
-	return e.ReadWorkspaceFile(name)
+	content, err := e.Sandbox.Workdir.ReadFile(name)
+	if err != nil {
+		if errors.Is(err, os.ErrNotExist) {
+			return ""
+		} else {
+			e.T.Fatal(err)
+		}
+	}
+	return string(content)
 }
 
 // RegexpSearch returns the starting position of the first match for re in the
@@ -207,7 +217,7 @@ func (e *Env) OrganizeImports(name string) {
 // ApplyQuickFixes processes the quickfix codeAction, calling t.Fatal on any error.
 func (e *Env) ApplyQuickFixes(path string, diagnostics []protocol.Diagnostic) {
 	e.T.Helper()
-	loc := protocol.Location{URI: e.Sandbox.Workdir.URI(path)} // zero Range => whole file
+	loc := e.Sandbox.Workdir.EntireFile(path)
 	if err := e.Editor.ApplyQuickFixes(e.Ctx, loc, diagnostics); err != nil {
 		e.T.Fatal(err)
 	}
@@ -224,7 +234,7 @@ func (e *Env) ApplyCodeAction(action protocol.CodeAction) {
 // GetQuickFixes returns the available quick fix code actions.
 func (e *Env) GetQuickFixes(path string, diagnostics []protocol.Diagnostic) []protocol.CodeAction {
 	e.T.Helper()
-	loc := protocol.Location{URI: e.Sandbox.Workdir.URI(path)} // zero Range => whole file
+	loc := e.Sandbox.Workdir.EntireFile(path)
 	actions, err := e.Editor.GetQuickFixes(e.Ctx, loc, diagnostics)
 	if err != nil {
 		e.T.Fatal(err)
@@ -233,6 +243,7 @@ func (e *Env) GetQuickFixes(path string, diagnostics []protocol.Diagnostic) []pr
 }
 
 // Hover in the editor, calling t.Fatal on any error.
+// It may return (nil, zero) even on success.
 func (e *Env) Hover(loc protocol.Location) (*protocol.MarkupContent, protocol.Location) {
 	e.T.Helper()
 	c, loc, err := e.Editor.Hover(e.Ctx, loc)
@@ -355,13 +366,13 @@ func (e *Env) ExecuteCodeLensCommand(path string, cmd command.Command, result in
 	var lens protocol.CodeLens
 	var found bool
 	for _, l := range lenses {
-		if l.Command.Command == cmd.ID() {
+		if l.Command.Command == cmd.String() {
 			lens = l
 			found = true
 		}
 	}
 	if !found {
-		e.T.Fatalf("found no command with the ID %s", cmd.ID())
+		e.T.Fatalf("found no command with the ID %s", cmd)
 	}
 	e.ExecuteCommand(&protocol.ExecuteCommandParams{
 		Command:   lens.Command.Command,
@@ -421,7 +432,7 @@ func (e *Env) StartProfile() (stop func() string) {
 		e.T.Fatal(err)
 	}
 	params := &protocol.ExecuteCommandParams{
-		Command:   command.StartProfile.ID(),
+		Command:   command.StartProfile.String(),
 		Arguments: args,
 	}
 	var result command.StartProfileResult
@@ -433,7 +444,7 @@ func (e *Env) StartProfile() (stop func() string) {
 			e.T.Fatal(err)
 		}
 		stopParams := &protocol.ExecuteCommandParams{
-			Command:   command.StopProfile.ID(),
+			Command:   command.StopProfile.String(),
 			Arguments: stopArgs,
 		}
 		var result command.StopProfileResult
@@ -533,12 +544,17 @@ func (e *Env) AcceptCompletion(loc protocol.Location, item protocol.CompletionIt
 	}
 }
 
-// CodeAction calls textDocument/codeAction for the given path, and calls
-// t.Fatal if there are errors.
-func (e *Env) CodeAction(path string, diagnostics []protocol.Diagnostic) []protocol.CodeAction {
+// CodeActionForFile calls textDocument/codeAction for the entire
+// file, and calls t.Fatal if there were errors.
+func (e *Env) CodeActionForFile(path string, diagnostics []protocol.Diagnostic) []protocol.CodeAction {
+	return e.CodeAction(e.Sandbox.Workdir.EntireFile(path), diagnostics, protocol.CodeActionUnknownTrigger)
+}
+
+// CodeAction calls textDocument/codeAction for a selection,
+// and calls t.Fatal if there were errors.
+func (e *Env) CodeAction(loc protocol.Location, diagnostics []protocol.Diagnostic, trigger protocol.CodeActionTriggerKind) []protocol.CodeAction {
 	e.T.Helper()
-	loc := protocol.Location{URI: e.Sandbox.Workdir.URI(path)} // no Range => whole file
-	actions, err := e.Editor.CodeAction(e.Ctx, loc, diagnostics)
+	actions, err := e.Editor.CodeAction(e.Ctx, loc, diagnostics, trigger)
 	if err != nil {
 		e.T.Fatal(err)
 	}
