@@ -11,15 +11,12 @@
 // for the most up-to-date documentation.
 package main // import "golang.org/x/tools/gopls"
 
-//go:generate go run doc/generate.go
-
 import (
 	"context"
 	"os"
 
 	"golang.org/x/telemetry"
 	"golang.org/x/tools/gopls/internal/cmd"
-	"golang.org/x/tools/gopls/internal/hooks"
 	versionpkg "golang.org/x/tools/gopls/internal/version"
 	"golang.org/x/tools/internal/tool"
 )
@@ -29,7 +26,11 @@ var version = "" // if set by the linker, overrides the gopls version
 func main() {
 	versionpkg.VersionOverride = version
 
-	telemetry.Start(telemetry.Config{ReportCrashes: true})
+	telemetry.Start(telemetry.Config{
+		ReportCrashes: true,
+		Upload:        true,
+	})
+
 	ctx := context.Background()
-	tool.Main(ctx, cmd.New(hooks.Options), os.Args[1:])
+	tool.Main(ctx, cmd.New(), os.Args[1:])
 }
